// Copyright (C) 2010, 2011 GlavSoft LLC.
// All rights reserved.
//
//-------------------------------------------------------------------------
// This file is part of the TightVNC software.  Please visit our Web site:
//
//                       http://www.tightvnc.com/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//-------------------------------------------------------------------------
//

package com.glavsoft.rfb.encoding.decoder;

import com.glavsoft.drawing.Renderer;
import com.glavsoft.exceptions.TransportException;
import com.glavsoft.transport.Transport.Reader;
import com.glavsoft.rfb.client.FBCrypt; // EGAWA

public class RawDecoder extends Decoder {
	private static RawDecoder instance = new RawDecoder();
	public static RawDecoder getInstance() {
		return instance;
	}
	private RawDecoder() { /*empty*/ }

	@Override
	public void decode(Reader reader,
			Renderer renderer, FramebufferUpdateRectangle rect) throws TransportException {
		decode(reader, renderer, rect.x, rect.y, rect.width, rect.height);
	}

	public void decode(Reader reader, Renderer renderer, int x, int y,
			int width, int height) throws TransportException {
		int length = width * height * renderer.getBytesPerPixel();
		byte [] bytes = ByteBuffer.getInstance().getBuffer(length);
		reader.readBytes(bytes, 0, length);

		long start = System.nanoTime(); /* VFB-DECRYPT-TIME */

		// EGAWA
		//if ( (x==0 && y==0) || (x==1&&y==110) ) {
		//if ( (x==160) ) {
		//System.out.printf("x:%d y:%d w:%d h:%d len:%d getbytesPerPixel:%d\n"
		//		,x,y,width,height,length,renderer.getBytesPerPixel());
		//System.out.printf("bytes[0]:%02x\n",bytes[0]);
		//System.out.printf("bytes[1]:%02x\n",bytes[1]);
		//System.out.printf("bytes[2]:%02x\n",bytes[2]);
		//System.out.printf("bytes[3]:%02x\n",bytes[3]);
		//System.out.printf("bytes[4]:%02x\n",bytes[4]);
		//System.out.printf("bytes[5]:%02x\n",bytes[5]);
		//System.out.printf("bytes[6]:%02x\n",bytes[6]);
		//System.out.printf("bytes[7]:%02x\n",bytes[7]);
		//}
		/*
		System.out.printf("bytes[%d]:%02x\n",length-4,bytes[length-4]);
		System.out.printf("bytes[%d]:%02x\n",length-3,bytes[length-3]);
		System.out.printf("bytes[%d]:%02x\n",length-2,bytes[length-2]);
		System.out.printf("bytes[%d]:%02x\n",length-1,bytes[length-1]);
		*/

		/* naonishi_Decrypt ON / OFF */
		if ( FBCrypt.SCREEN_DECRYPT ) {
			renderer.drawBytesCrypt(bytes, x, y, width, height);
		} else {
			renderer.drawBytes(bytes, x, y, width, height);
		}

		if ( FBCrypt.OVERHEAD ) {
			long end = System.nanoTime();
			double overhead = (double)(end-start)/1000000;
			System.out.printf("screen_overhead:%.4f w:%d h:%d\n",overhead,width,height);
		}

		/*
		if(x!=256 && y!=336 && width==16 && height==16){//x==0 && y==0 && width==800 && height==600){
			start = System.nanoTime();

			renderer.drawBytesCrypt(bytes, x, y, width, height); // Decrypt ON
			//renderer.drawBytes(bytes, x, y, width, height); // Decrypt OFF

			long stop = System.nanoTime();
			//System.out.printf("normal-decrypt\n");
			System.out.println(stop-start);
		}else{
			//System.out.printf("x:%d y:%d width:%d height:%d\n", x, y, width, height);
			renderer.drawBytesCrypt(bytes, x, y, width, height);
		}*/
	}

}
