/* aes_fbcrypt.c */

#include <xen/string.h>
#include <xen/lib.h>
#include <crypto/fbcrypt/cyassl/cyassl.h>

#define MYDEBUG_AES 0

/* secret_key */
const byte ctrKey[] = 
{
    0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,
    0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f
};

/* IV */
const byte ctrIv[] =
{
    0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,
    0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f
};

#if MYDEBUG_AES
//static void MYDEBUG_info(const char* msg, const byte *data, int num)
//{
//	int i;
//	printk("%s	",msg);
//	for ( i=0; i<num; i++ ) {
//		printk("%02x ",data[i]);
//	}
//	printk("\n");
//}
#endif


static void divide_int_to_byte(unsigned int input, byte block[])
{
	block[0] = input & 0x00ff;
	block[1] = (input & 0xff00)>>8;
}

static unsigned int combine_byte_to_int(byte block[])
{
	unsigned int data;
	unsigned int tmp[2];
	tmp[0] = block[0] & 0x00ff;
	tmp[1] = (block[1]<<8) & 0xff00;
	data = ( tmp[0] | tmp[1] ) & 0xffff;
	return data;
}

#if MYDEBUG_AES 
static void log(unsigned int input, unsigned int encrypted, unsigned int decrypted)
{
	printk("\nmode: AES_CTR -------------------------\n");
	printk("transmited:0x%08x -> enc:0x%04x -> dec:0x%04x\n",input,encrypted,decrypted);
	//printk("encrypted: %d(0x%04x)\n",encrypted,encrypted);
	//printk("decrypted: %d(0x%04x)\n",decrypted,decrypted);
}
#endif

static void is_key_exchange(AES_kbd *obj,int reset_flag)
{
	if ( reset_flag == 1 || obj->key_exchange == 0 ) {
		printk("\n============== INIT AES Internal_State ==============\n");
		AesSetKey(&obj->aes, ctrKey, AES_BLOCK_SIZE, ctrIv, AES_ENCRYPTION);
		obj->key_exchange = 1;
		obj->used = AES_BLOCK_SIZE;
	}
}

unsigned int AES_kbd_decrypt(dominfo_p dominfo, unsigned int input,int reset_key)
{
	unsigned int ioring_write_data = 0;
    byte encrypted[2] = {0};
    byte decrypt[2] = {0};

	AES_kbd *obj = AES_prepare(dominfo);
	is_key_exchange(obj,reset_key);
	divide_int_to_byte(input,encrypted);
	AesCtrEncrypt(&obj->aes, decrypt, encrypted,2,&obj->used,obj->counter);
	ioring_write_data = combine_byte_to_int(decrypt);	

	#if MYDEBUG_AES
	log(input,combine_byte_to_int(encrypted),ioring_write_data);
	#endif

	return ioring_write_data;
}



int aes_test()
{
#if 0
    Aes enc;
    Aes dec;

    const byte msg[] = { /* "Now is the time for all " w/o trailing 0 */
        0x6e,0x6f,0x77,0x20,0x69,0x73,0x20,0x74,
        0x68,0x65,0x20,0x74,0x69,0x6d,0x65,0x20,
        0x66,0x6f,0x72,0x20,0x61,0x6c,0x6c,0x20
    };

    const byte verify[] = 
    {
        0x95,0x94,0x92,0x57,0x5f,0x42,0x81,0x53,
        0x2c,0xcc,0x9d,0x46,0x77,0xa2,0x33,0xcb
    };

    byte key[] = "0123456789abcdef";  /* align */
    byte iv[]  = "1234567890abcdef";  /* align */

    byte cipher[AES_BLOCK_SIZE * 4];
    byte plain [AES_BLOCK_SIZE * 4];

    AesSetKey(&enc, key, AES_BLOCK_SIZE, iv, AES_ENCRYPTION);
    AesSetKey(&dec, key, AES_BLOCK_SIZE, iv, AES_DECRYPTION);

    AesCbcEncrypt(&enc, cipher, msg,   AES_BLOCK_SIZE);
    AesCbcDecrypt(&dec, plain, cipher, AES_BLOCK_SIZE);

	#if MYDEBUG_AES
	printk("mode: CBC -------------------------\n");
	MYDEBUG_info("msg:",msg,AES_BLOCK_SIZE);
	MYDEBUG_info("cipher:",cipher,AES_BLOCK_SIZE);
	MYDEBUG_info("plain:",plain,AES_BLOCK_SIZE);
	#endif

    if (memcmp(plain, msg, AES_BLOCK_SIZE))
        return -60;

    if (memcmp(cipher, verify, AES_BLOCK_SIZE))
        return -61;

#ifdef CYASSL_AES_COUNTER
    {
        const byte ctrKey[] = 
        {
            0x2b,0x7e,0x15,0x16,0x28,0xae,0xd2,0xa6,
            0xab,0xf7,0x15,0x88,0x09,0xcf,0x4f,0x3c
        };

        const byte ctrIv[] =
        {
            0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,
            0xf8,0xf9,0xfa,0xfb,0xfc,0xfd,0xfe,0xff
        };


        const byte ctrPlain[] =
        {
            0x6b,0xc1,0xbe,0xe2,0x2e,0x40,0x9f,0x96,
            0xe9,0x3d,0x7e,0x11,0x73,0x93,0x17,0x2a,
            0xae,0x2d,0x8a,0x57,0x1e,0x03,0xac,0x9c,
            0x9e,0xb7,0x6f,0xac,0x45,0xaf,0x8e,0x51,
            0x30,0xc8,0x1c,0x46,0xa3,0x5c,0xe4,0x11,
            0xe5,0xfb,0xc1,0x19,0x1a,0x0a,0x52,0xef,
            0xf6,0x9f,0x24,0x45,0xdf,0x4f,0x9b,0x17,
            0xad,0x2b,0x41,0x7b,0xe6,0x6c,0x37,0x10
        };

        const byte ctrCipher[] =
        {
            0x87,0x4d,0x61,0x91,0xb6,0x20,0xe3,0x26,
            0x1b,0xef,0x68,0x64,0x99,0x0d,0xb6,0xce,
            0x98,0x06,0xf6,0x6b,0x79,0x70,0xfd,0xff,
            0x86,0x17,0x18,0x7b,0xb9,0xff,0xfd,0xff,
            0x5a,0xe4,0xdf,0x3e,0xdb,0xd5,0xd3,0x5e,
            0x5b,0x4f,0x09,0x02,0x0d,0xb0,0x3e,0xab,
            0x1e,0x03,0x1d,0xda,0x2f,0xbe,0x03,0xd1,
            0x79,0x21,0x70,0xa0,0xf3,0x00,0x9c,0xee
        };

        AesSetKey(&enc, ctrKey, AES_BLOCK_SIZE, ctrIv, AES_ENCRYPTION);
        /* Ctr only uses encrypt, even on key setup */
        AesSetKey(&dec, ctrKey, AES_BLOCK_SIZE, ctrIv, AES_ENCRYPTION);

        AesCtrEncrypt(&enc, cipher, ctrPlain, AES_BLOCK_SIZE*4);
        AesCtrEncrypt(&dec, plain, cipher, AES_BLOCK_SIZE*4);
	
		#if MYDEBUG_AES
		printk("\nmode: CTR -------------------------\n");
		MYDEBUG_info("msg:",ctrPlain,AES_BLOCK_SIZE*4);
		MYDEBUG_info("cipher:",cipher,AES_BLOCK_SIZE*4);
		MYDEBUG_info("plain:",plain,AES_BLOCK_SIZE*4);
		#endif

        if (memcmp(plain, ctrPlain, AES_BLOCK_SIZE*4))
            return -66;

        if (memcmp(cipher, ctrCipher, AES_BLOCK_SIZE*4))
            return -67;
    }
#endif /* CYASSL_AES_COUNTER */


#endif
    return 0;
}
