#include <xen/lib.h>
#include <crypto/fbcrypt/cyassl/cyassl.h>

unsigned int RC4_key_decrypt(dominfo_p p, uint32_t input)
{
	unsigned char encrypt[2] = {0};	
	unsigned char decrypt[2] = {0};	
	unsigned int join[2] = {0};
	unsigned int write_data = 0;
	RC4_table rc4t;

	/* Get domain_info for RC4 */
	RC4_get_rc4t(p,&rc4t);

	/* flag==0 なら内部状態初期化 */
	if ( rc4t.ksa_flag == 0 ) {
		printk("============== INIT RC4 Internal_State ==============\n");
		RC4_set_key(&rc4t.rc4key,rc4t.secret_len,(unsigned char*)rc4t.secret_key);
		RC4_initflag_on(p);
	}

	/* divide */
	encrypt[0] = input & 0x00ff;
	encrypt[1] = (input & 0xff00)>>8;

	#if DEBUG_VKBD_INFO
	printk("--------------------------------\n");
	printk("ENCRYPT:%d(0x%08x) \n",input,input);
	printk("encrypt[0]:%d(0x%02x)\n",encrypt[0],encrypt[0]);
	printk("encrypt[1]:%d(0x%02x)\n",encrypt[1],encrypt[1]);
	#endif

	#if DEBUG_RC4_INFO
	printk("RC4 INFO ***********************\n");
	printk("[Before] x:%d y:%d\n",rc4t.rc4key.x, rc4t.rc4key.y);
	#endif

	/* RC4 decrypt --------------------*/
	RC4(&rc4t.rc4key,2,encrypt,decrypt);

	/* combine */
	join[0] = decrypt[0] & 0x00ff;
	join[1] = decrypt[1];
	join[1] = (join[1]<<8) & 0xff00;
	write_data = ( join[0] | join[1] ) & 0xffff;

	#if DEBUG_RC4_INFO
	printk("[After]  x:%d y:%d\n",rc4t.rc4key.x, rc4t.rc4key.y);
	printk("DECRYPT:%d(0x%08x) \n",write_data,write_data);
	printk("********************************\n");
	#endif

	/* sync domain_info & RC4_state */
	RC4_InternalState_Sync(p, &rc4t);

	return write_data;
}

