#include <asm/system.h>         /* hypercall */
#include <asm/current.h>        // current
#include <asm/page.h>           // virt_to_mfn
#include <xen/domain_page.h>    // map_domain_page
#include <xen/mm.h>             // alloc_domheap_pages
#include <xen/sched.h>          // struct domain
#include <xen/xmalloc.h>        // xmalloc_bytes
#include <public/io/fbif.h>     // struct xenfb_page
#include "RC5_48.h"

typedef unsigned char u1;
typedef unsigned long u4;

#define ROTL24(X, C) (((X)<<(C)) | ((X)>>(24-(C))))
#define ROTR24(X, C) (((X)>>(C)) | ((X)<<(24-(C))))

/*
typedef struct {
    u4 *xk;
    int nr;
}rc5_ctx;

void rc5_init(rc5_ctx *, int);
void rc5_destroy(rc5_ctx *);
void rc5_key(rc5_ctx *, u1 *, int, int, int);
void rc5_encrypt(rc5_ctx *, u4 *, int);
void rc5_decrypt(rc5_ctx *, u4 *, int);
*/

void rc5_destroy(rc5_ctx *c)
{
    int i;

    for(i=0; i<(c->nr)*2+2; i++)
        c->xk[i] = 0;

    xfree((void *)c->xk);
}

void rc5_init(rc5_ctx *c, int rounds)
{
    c->nr = rounds;
    c->xk = (unsigned int *)xmalloc_bytes(sizeof(unsigned int)*(rounds*2+2));
}

void rc5_encrypt(rc5_ctx *c, u4 *data, int blocks, int x)
{
    u4 *d;
    unsigned int *sk;
    int h, i, rc;

    d = data;
    sk = (c->xk) + 2;

    for(h=0; h<blocks; h++){
        d[0] += (c->xk[0] ^ (((x+c->nr)*c->nr)&0x000ffffff));
        d[0] &= 0x000ffffff;
        d[1] += (c->xk[1] ^ (((x+c->nr)*c->nr)&0x000ffffff));
        d[1] &= 0x000ffffff;
        for(i=0; i<c->nr*2; i+=2){
            d[0] ^= d[1];
            rc = d[1] & 23;
            d[0] = ROTL24(d[0], rc);
            d[0] &= 0x000ffffff;
            d[0] += sk[i];
            d[0] &= 0x000ffffff;
            d[1] ^= d[0];
            rc = d[0] & 23;
            d[1] = ROTL24(d[1], rc);
            d[1] &= 0x000ffffff;
            d[1] += sk[i+1];
            d[1] &= 0x000ffffff;
        }
        d += 2;
    }
}


void rc5_decrypt(rc5_ctx *c, u4 *data, int blocks, int x)
{
    u4 *d;
    unsigned int *sk;
    int h, i, rc;

    d = data;
    sk = (c->xk) + 2;

    for(h=0; h<blocks; h++){
        for(i=c->nr*2-2; i>=0; i-=2){
            d[1] -= sk[i+1];
            d[1] &= 0x000ffffff;
            rc = d[0] & 23;
            d[1] = ROTR24(d[1], rc);
            d[1] &= 0x000ffffff;
            d[1] ^= d[0];
            d[0] -= sk[i];    
            d[0] &= 0x000ffffff;
            rc = d[1] & 23;
            d[0] = ROTR24(d[0], rc);
            d[0] &= 0x000ffffff;
            d[0] ^= d[1];
        }
        d[0] -= (c->xk[0] ^ (((x+c->nr)*c->nr)&0x000ffffff));
        d[0] &= 0x000ffffff;
        d[1] -= (c->xk[1] ^ (((x+c->nr)*c->nr)&0x000ffffff));
        d[1] &= 0x000ffffff;
        d += 2;
    }
}


void rc5_key(rc5_ctx *c, u1 *key, int keylen, int y)
{
    unsigned int *pk, A, B;
    int xk_len, pk_len, i, j, k, num_steps, rc;

    xk_len = c->nr*2 + 2;
    pk_len = keylen / 4;
    if((keylen%4) != 0)
        pk_len += 1;

    pk = (unsigned int *)xmalloc_bytes(sizeof(unsigned int)*pk_len);
    if(pk == NULL){
        printk("An error occurred! in RC5_48\n");
    }

    
    for(i=0,j=0; i<pk_len; i++,j+=4){
        pk[i] = 0;
        for(k=0; k<4; k++){
            pk[i] <<= 8;
            if((j+k) < keylen){
                pk[i] |= key[j+k] &0x0ff;
                pk[i] ^= (y+keylen)*keylen;
            }
        }
    }


    c->xk[0] = 0x00b7e151;
    for(i=1; i<xk_len; i++){
        c->xk[i] = (c->xk[i-1] + 0x009e3779) & 0x000ffffff;
    }
    
    if(pk_len > xk_len)
        num_steps = 3 * pk_len;
    else
        num_steps = 3 * xk_len;

    A = B = 0;
    for(i=0; i<num_steps; i++){
        A = c->xk[i % xk_len] = ROTL24(c->xk[i % xk_len] + A + B, 3) & 0x00ffffff;
        rc = (A+B) & 23;
        B = pk[i % pk_len] = ROTL24((pk[i % pk_len] + A + B) & 0x00ffffff, rc) & 0x00ffffff;
    }

    for(i = 0; i<pk_len; i++)
        pk[i] = 0;
    
    xfree((void *)pk);
}
/*
int main(int argc, char* argv[])
{
    int i, keylen, words = 30;
    rc5_ctx c;
    u4 data[words];
    //u1 key[] = {0x08,0x23,0x70,0x61,0x43,0xfa,0x33,0x12,0xab,0xef,0xfd,0x43,0xcd,0x43,0x43,0xdf,0xac};
    u1 *key_tmp, *key = "08237061";

    for(key_tmp=key, keylen=0; *key_tmp != '\0'; key_tmp++)
        keylen++;
    
    for(i=0; i<keylen; i++)
        printf("%c %04x\n", key[i], key[i]);

    printf("---------------------------------------------\n");
    for(i=0; i<words; i++){
        data[i] = i;
    }

    rc5_init(&c, 20);
    rc5_key(&c, key, keylen);

    rc5_encrypt(&c, data, words/2);
    printf("Encryptions:\n");
    for(i=0; i<words; i+=2){
        printf("Block %01d = %08lx %08lx\n",
                i/2, data[i], data[i+1]);
    }
    
    rc5_decrypt(&c, data, words/2);
    printf("Decryptions:\n");
    for(i=0; i<words; i+=2){
        printf("Block %01d = %08lx %08lx\n",
                i/2, data[i], data[i+1]);
    }

    rc5_destroy(&c);
    
    return 0;
}
*/
