/*-----------------------------------------------
 * xen/fbcrypt/domain_info.c
 *
 * 各ドメインごとにFBCrypt用の情報を保存・参照
 * VMMが保存している情報は関数越しに呼ぶこと
 *
 * ドメイン情報は連結リストで管理
 * ---------------------------------------------*/

#include <xen/types.h>
#include <xen/domain_page.h>
#include <xen/string.h>

#include <crypto/fbcrypt/cyassl/cyassl.h>
#include <crypto/fbcrypt/configure.h>

//int init_vfb_naonishi(int domid); // naonishi/fbcrypt-v.c

/* RC4 秘密鍵 */
static char *RC4_SECRET_KEY = "nanohasan";

typedef struct fbcrypt_domain_list {
	int domid;
	int type; /* HVM:0 or PV:1 */
	unsigned long start_mfn;
	unsigned long store_mfn;
	xs_mfn_flag vkbd;
	xs_mfn_flag vfb;
	xs_mfn_flag vif_tx;
	xs_mfn_flag vif_rx;
	xs_mfn_flag vbd;

	/* XenStore Ring */
	unsigned int cons;
	unsigned int prod;
	unsigned int avail;
	unsigned int cnt_xs_write;
	unsigned int cnt_xs_all;

	/* RC4_table */
	RC4_table rc4t;

	/* AES */
	AES_kbd aes_kbd;

	struct fbcrypt_domain_list *next;
} fbc_dom_t; 

/* dummy list_head */
static fbc_dom_t list_head; 
static fbc_dom_t *head = &list_head;


static void print_domain_info(fbc_dom_t *p)
{
	printk("==============================\n");
	printk("   Domain:%d VMM_INFO\n",p->domid);
	printk("==============================\n");
	switch(p->type) {
		case HVM_DOMAIN: printk("type: HVM\n"); break;
		case PV_DOMAIN:  printk("type: PV\n");  break;
	}
	printk("start_mfn:%ld\n",p->start_mfn);
	printk("store_mfn:%ld\n",p->store_mfn);
	#if DEBUG_FBCRYPT_PV > 30
	printk("cnt_xs_write:%d\n",p->cnt_xs_write);
	printk("cnt_xs_all:%d\n",p->cnt_xs_all);
	#endif


	printk("[VFB]  mfn:%ld\n",p->vfb.mfn); 
	#if DEBUG_FBCRYPT_PV > 30
	printk("header_flag:%d\n",p->vfb.header_flag);
	printk("mem_avail:%d\n",p->vfb.mem_avail);
	printk("mem_cons:%d\n",p->vfb.mem_cons);
	printk("mem_prod:%d\n",p->vfb.mem_prod);
	#endif

	printk("[VKBD] mfn:%ld\n",p->vkbd.mfn);
	#if DEBUG_FBCRYPT_PV > 30
	printk("header_flag:%d\n",p->vkbd.header_flag);
	printk("mem_avail:%d\n",p->vkbd.mem_avail);
	printk("mem_cons:%d\n",p->vkbd.mem_cons);
	printk("mem_prod:%d\n",p->vkbd.mem_prod);
	#endif

	printk("[VBD]  mfn:%ld\n",p->vbd.mfn);
	#if DEBUG_FBCRYPT_PV > 30
	printk("header_flag:%d\n",p->vbd.header_flag);
	printk("mem_avail:%d\n",p->vbd.mem_avail);
	printk("mem_cons:%d\n",p->vbd.mem_cons);
	printk("mem_prod:%d\n",p->vbd.mem_prod);
	#endif

	printk("[VIF-TX] tx-ring-ref:%ld\n",p->vif_tx.mfn);
	#if DEBUG_FBCRYPT_PV > 30
	printk("header_flag:%d\n",p->vif_tx.header_flag);
	printk("mem_avail:%d\n",p->vif_tx.mem_avail);
	printk("mem_cons:%d\n",p->vif_tx.mem_cons);
	printk("mem_prod:%d\n",p->vif_tx.mem_prod);
	#endif

	printk("[VIF-RX] rx-ring-ref:%ld\n",p->vif_rx.mfn);
	#if DEBUG_FBCRYPT_PV > 30
	printk("header_flag:%d\n",p->vif_rx.header_flag);
	printk("mem_avail:%d\n",p->vif_rx.mem_avail);
	printk("mem_cons:%d\n",p->vif_rx.mem_cons);
	printk("mem_prod:%d\n",p->vif_rx.mem_prod);
	#endif

	printk("\nRC4 INFO -----------------\n");
	printk("rc4key.x: %d\n",p->rc4t.rc4key.x);
	printk("rc4key.y: %d\n",p->rc4t.rc4key.y);
	printk("ksa_flag: %d\n",p->rc4t.ksa_flag);
	printk("secret_key: %s\n",p->rc4t.secret_key);
	printk("secret_len: %d\n\n",p->rc4t.secret_len);

}

/* create-addlist-init まで */
int Domain_add_list(int domid, V_TYPE type)
{
	fbc_dom_t *new;

	new = (fbc_dom_t*)xmalloc(fbc_dom_t);
	if ( new == NULL ) {
		printk("ERROR: xmalloc fail\n");
		return 1;
	}

	/* add list */
	new->next = head->next;
	head->next = new;

	new->domid = domid;
	new->type = type;
	new->start_mfn = 0;
	new->store_mfn = 0;

	new->cnt_xs_write = 0;
	new->cnt_xs_all = 0;

	/* VKB */
	new->vkbd.mfn = 0;
	new->vkbd.header_flag = 0;
	new->vkbd.mem_avail = 0;
	new->vkbd.mem_cons = 0;
	new->vkbd.mem_prod = 0;

	/* VFB */
	new->vfb.mfn = 0;
	new->vfb.header_flag = 0;
	new->vfb.mem_avail = 0;
	new->vfb.mem_cons = 0;
	new->vfb.mem_prod = 0;

	/* VIF TX */
	new->vif_tx.mfn = 0;
	new->vif_tx.header_flag = 0;
	new->vif_tx.mem_avail = 0;
	new->vif_tx.mem_cons = 0;
	new->vif_tx.mem_prod = 0;

	/* VIF RX */
	new->vif_rx.mfn = 0;
	new->vif_rx.header_flag = 0;
	new->vif_rx.mem_avail = 0;
	new->vif_rx.mem_cons = 0;
	new->vif_rx.mem_prod = 0;

	/* VBD */
	new->vbd.mfn = 0;
	new->vbd.header_flag = 0;
	new->vbd.mem_avail = 0;
	new->vbd.mem_cons = 0;
	new->vbd.mem_prod = 0;

	/* RC4 */
	new->rc4t.rc4key.x = 0;
	new->rc4t.rc4key.y = 0;
	new->rc4t.ksa_flag = 0;
	safe_strcpy(new->rc4t.secret_key,RC4_SECRET_KEY);
	new->rc4t.secret_len = strlen(RC4_SECRET_KEY);

	/* AES */
	new->aes_kbd.key_exchange = 0;
	new->aes_kbd.used = 0;
	//safe_strcpy(new->aes_kbd.secretkey,ctrKey);
	//safe_strcpy(new->aes_kbd.iv,ctriv);


	return 0;
}

int Domain_delete_list(int domid)
{
	fbc_dom_t *p = head->next;
	fbc_dom_t *back = NULL;
	
	/* DomainUの存在確認 */
	if ( p == NULL ) {
		printk("Delete_ERROR: No exit DomainU\n");
		return 1;
	}

	/* 目的のdomidのnodeを探索 */
	while ( p->domid != domid ) {
		back = p; /* domidのひとつ前の要素を保持 */
		p = p->next;
		if ( p == NULL ) {
			printk("Delete_ERROR: Invalid domain_id\n");
			return 1;
		}
	}

	back->next = p->next;
	xfree(p);
	
	return 0;
}

int Domain_ref_list(int domid)
{
	fbc_dom_t *p = head->next;

	if ( p == NULL ) {
		printk("ERROR: Domain_ref_list: No exit DomainU\n");
		return 1;
	}

	while ( p->domid != domid ) {
		p = p->next;
		if ( p == NULL ) {
			printk("ERROR: Domain_ref_list: Invalid domain_id\n");
			return 1;
		}
	}

	print_domain_info(p);

	return 0;
}

int Domain_regist_info(int domid, unsigned long mfn, XS_DATA cmd)
{
	fbc_dom_t *p = head->next;

	if ( p == NULL ) {
		printk("ERROR: Domain_regist_info: No exit DomainU\n");
		return 1;
	}

	while ( p->domid != domid ) {
		p = p->next;
		if ( p == NULL ) {
			printk("ERROR: Domain_regist_info: Invalid domain_id\n");
			return 1;
		}
	}

	switch(cmd) {
		case VMM_START: p->start_mfn = mfn; break;
		case VMM_STORE: p->store_mfn = mfn; break;
		case VMM_VKBD:  p->vkbd.mfn = mfn; 	break; 
		case VMM_VFB:   p->vfb.mfn = mfn; 	
						//init_vfb_naonishi(domid);
						break;
		case VMM_VIF_TX:p->vif_tx.mfn = mfn;break;
		case VMM_VIF_RX:p->vif_rx.mfn = mfn;break;
		case VMM_VBD:	p->vbd.mfn = mfn; 	break;
	}
	return 0;
}

int Domain_ref_all(void)
{
	int total = 0;
	fbc_dom_t *p = head->next;

	if ( p == NULL ) {
		printk("ERROR: Domain_ref_all: No exist DomainU\n");
		return 1;
	}

	do {
		print_domain_info(p);
		p = p->next;
		total++;
	} while ( p != NULL );

	printk("Domain_TOTAL:%d\n",total);
	return 0;
}



unsigned long Domain_ref_mfn(int domid, XS_DATA cmd)
{
	fbc_dom_t *p = head->next;

	if ( p == NULL ) {
		printk("ERROR: Domain_ref_mfn: No exit DomainU\n");
		printk("domid:%d cmd:%d\n",domid,cmd);
		return 1;
	}

	while ( p->domid != domid ) {
		p = p->next;
		if ( p == NULL ) {
			printk("ERROR: Domain_ref_mfn: Invalid domain_id\n");
			return 1;
		}
	}

	switch(cmd) {
		case VMM_START: return p->start_mfn; break;
		case VMM_STORE: return p->store_mfn; break;
		case VMM_VKBD:  return p->vkbd.mfn;  break; 
		case VMM_VFB:   return p->vfb.mfn;	 break;
		case VMM_VIF_TX:return p->vif_tx.mfn;break;
		case VMM_VIF_RX:return p->vif_rx.mfn;break;
		case VMM_VBD:   return p->vbd.mfn;	 break;
	}
	return 0;
}

void *Domain_mfn2map_page(int domid, XS_DATA cmd)
{
	fbc_dom_t *p = head->next;
	void *v_adr = NULL;

	if ( p == NULL ) {
		printk("ERROR: Domain_mfn2map_page: No exit DomainU\n");
		return NULL;
	}

	while ( p->domid != domid ) {
		p = p->next;
		if ( p == NULL ) {
			printk("ERROR: Domain_mfn2map_page: Invalid domain_id\n");
			return NULL;
		}
	}


	switch(cmd) {
		case VMM_START: 
			if ( p->start_mfn != 0 ) v_adr = map_domain_page(p->start_mfn); break;
		case VMM_STORE: 
			if ( p->store_mfn != 0 ) v_adr = map_domain_page(p->store_mfn); break;
		case VMM_VKBD:
			if ( p->vkbd.mfn != 0 ) v_adr = map_domain_page(p->vkbd.mfn); break;
		case VMM_VFB:
			if ( p->vfb.mfn != 0 ) v_adr = map_domain_page(p->vfb.mfn); break;
		case VMM_VIF_TX:
			if ( p->vif_tx.mfn != 0 ) v_adr = map_domain_page(p->vif_tx.mfn); break;
		case VMM_VIF_RX:
			if ( p->vif_rx.mfn != 0 ) v_adr = map_domain_page(p->vif_rx.mfn);	break;
		case VMM_VBD: 
			if ( p->vbd.mfn != 0 )  v_adr = map_domain_page(p->vbd.mfn);	break;
	}

	if ( v_adr == NULL ) {
		printk("Domain_mfn2map_page FAULT\n");
		return NULL;
	}

	return v_adr;
}

dominfo_p Domain_get_pointer(int domid)
{
	fbc_dom_t *p = head->next;

	if ( p == NULL ) {
		printk("ERROR: Domain_get_pointer: No existence DomainU\n");
		return NULL;
	}

	while ( p->domid != domid ) {
		p = p->next;
		if ( p == NULL ) {
			printk("ERROR: Domain_ref_mfn: Invalid domain_id\n");
			return NULL;
		}
	}

	return p;
}


/* XenStoreRing --------------------------------------------- */
int XS_header_on(dominfo_p p, XS_DATA cmd)
{
	switch(cmd) {
		case VMM_START: printk("Invalid cmd\n");return 1;break;
		case VMM_STORE: printk("Invalid_cmd\n");return 1;break;
		case VMM_VFB:   p->vfb.header_flag = 1; return 0;break;
		case VMM_VKBD:  p->vkbd.header_flag = 1;return 0;break; 
		case VMM_VBD:   p->vbd.header_flag = 1; return 0;break;
		case VMM_VIF_TX:p->vif_tx.header_flag = 1; return 0;break;
		case VMM_VIF_RX:p->vif_rx.header_flag = 1; return 0;break;
	}

	printk("header_flag_on:cmd:%d\n",cmd);
	return 0;
}

int XS_get_clue(dominfo_p p, XS_DATA cmd, xs_mfn_flag *clue)
{
	switch(cmd) {
		case VMM_START: printk("Invalid cmd\n");return 0; break;
		case VMM_STORE: printk("Invalid_cmd\n");return 0; break;
		case VMM_VKBD: 
				 clue->mfn = p->vkbd.mfn;
				 clue->header_flag = p->vkbd.header_flag;
				 clue->mem_avail = p->vkbd.mem_avail;
				 clue->mem_cons = p->vkbd.mem_cons;
				 clue->mem_prod = p->vkbd.mem_prod;
				 break;
					   	
		case VMM_VFB:  
				 clue->mfn = p->vfb.mfn;
				 clue->header_flag = p->vfb.header_flag;
				 clue->mem_avail = p->vfb.mem_avail;
				 clue->mem_cons = p->vfb.mem_cons;
				 clue->mem_prod = p->vfb.mem_prod;
				 break;
					   	
		case VMM_VIF_TX:  
				clue->mfn = p->vif_tx.mfn;
				clue->header_flag = p->vif_tx.header_flag;
				clue->mem_avail = p->vif_tx.mem_avail;
				clue->mem_cons 	= p->vif_tx.mem_cons;
				clue->mem_prod 	= p->vif_tx.mem_prod;
				break;

		case VMM_VIF_RX:  
				clue->mfn = p->vif_rx.mfn;
				clue->header_flag = p->vif_rx.header_flag;
				clue->mem_avail = p->vif_rx.mem_avail;
				clue->mem_cons 	= p->vif_rx.mem_cons;
				clue->mem_prod 	= p->vif_rx.mem_prod;
				break;
					   
		case VMM_VBD:  
				clue->mfn = p->vbd.mfn;
				clue->header_flag = p->vbd.header_flag;
				clue->mem_avail = p->vbd.mem_avail;
				clue->mem_cons 	= p->vbd.mem_cons;
				clue->mem_prod 	= p->vbd.mem_prod;
				break;
	}
	return 0;
}

int XS_save_memData(dominfo_p p, XS_DATA cmd,
 unsigned int cons, unsigned int prod, unsigned avail)
{
	switch(cmd) {
		case VMM_START: printk("Invalid cmd\n");return 0; break;
		case VMM_STORE: printk("Invalid_cmd\n");return 0; break;
		case VMM_VKBD:
				p->vkbd.mem_cons  = cons;
				p->vkbd.mem_prod  = prod;
				p->vkbd.mem_avail = avail;
				break;
		case VMM_VFB: 
				p->vfb.mem_cons  = cons;
				p->vfb.mem_prod  = prod;
				p->vfb.mem_avail = avail;
				break;
		case VMM_VIF_TX: 
				p->vif_tx.mem_cons  = cons;
				p->vif_tx.mem_prod  = prod;
				p->vif_tx.mem_avail = avail;
				break;
		case VMM_VIF_RX: 
				p->vif_rx.mem_cons  = cons;
				p->vif_rx.mem_prod  = prod;
				p->vif_rx.mem_avail = avail;
				break;
		case VMM_VBD: 
		  		p->vbd.mem_cons  = cons;
		  		p->vbd.mem_prod  = prod;
		  		p->vbd.mem_avail = avail;
			  	break;
	}
	return 0;
}

int XS_plus_cntAll(dominfo_p p)
{
	p->cnt_xs_all++;
	return 0;
}

int XS_plus_cntWrite(dominfo_p p)
{
	p->cnt_xs_write++;
	return 0;
}

/* for AES ----------------------------------------------- */
AES_kbd* AES_prepare(dominfo_p src)
{
	return &src->aes_kbd;
}

/* RC4 domain_info ----------------------------------------------- */
int RC4_get_rc4t(dominfo_p p, RC4_table *dst)
{
	dst->rc4key.x = p->rc4t.rc4key.x;
	dst->rc4key.y = p->rc4t.rc4key.y;
	memcpy(dst->rc4key.data,p->rc4t.rc4key.data,sizeof(dst->rc4key.data));
	dst->ksa_flag = p->rc4t.ksa_flag;
	safe_strcpy(dst->secret_key,p->rc4t.secret_key);
	dst->secret_len = p->rc4t.secret_len; 	  		

	return 0;
}

int RC4_initflag_on(dominfo_p p)
{
	p->rc4t.ksa_flag = 1;
	return 0;
}

int RC4_InternalState_Sync(dominfo_p p, RC4_table *sync)
{
	memcpy( &p->rc4t.rc4key,&sync->rc4key,sizeof(struct rc4_key_st) );
	return 0;
}

int RC4_reset(int domid)
{
	fbc_dom_t *p = head->next;

	if ( p == NULL ) {
		printk("ERROR: RC4_reset: No exit DomainU\n");
		return 1;
	}

	while ( p->domid != domid ) {
		p = p->next;
		if ( p == NULL ) {
			printk("ERROR: RC4_reset: Invalid domain_id\n");
			return 1;
		}
	}

	p->rc4t.ksa_flag = 0;

	return 0;
}
