/*
 * hvm.c: FBCrypt-HVM
 *
 */

#include <xen/lib.h>
#include <xen/time.h> // NOW()

#include <crypto/fbcrypt/hvm/hvm.h>
#include <crypto/fbcrypt/cyassl/cyassl.h>
#include <crypto/fbcrypt/keycode/keycode.h>

/* for keycode convert */
static VncState vs;
static int KBD_LAYOUT_initFlag=0;

/* for SHA1-MAC hvm-kbd */
struct save_kbd_hash {
	int domid;
	int seq;
	int sha1flag;
	unsigned int hash[5];
};
static struct save_kbd_hash save_hash;


static unsigned int ps2_decrypt(int domid, uint32_t encrypt, int resetKey)
{
	dominfo_p p = Domain_get_pointer(domid);

	/* init */
	if ( p == NULL ) {
		printk("============ CREATE HVM DOMAIN:%d INFO ============\n",domid);
		Domain_add_list(domid,HVM_DOMAIN);
		p = Domain_get_pointer(domid);
	}

	//return RC4_key_decrypt(p,encrypt);
	return AES_kbd_decrypt(p,encrypt,resetKey);
}


/* HC 41 */
asmlinkage int do_hvm_mac(FBCRYPT_info *fbc)
{
	save_hash.domid = fbc->domid;
	save_hash.seq = fbc->seq;
	save_hash.sha1flag = fbc->sha;
	save_hash.hash[0] = fbc->hash[0];
	save_hash.hash[1] = fbc->hash[1];
	save_hash.hash[2] = fbc->hash[2];
	save_hash.hash[3] = fbc->hash[3];
	save_hash.hash[4] = fbc->hash[4];
	return 0;
}

/* arch/x86/hvm/io.c hvm_io_assist */
static int hvm_decrypt(int domid, ioreq_t *p)
{
	unsigned int decrypt = 0;

	/* U32 data detail ------------------
	 * ex) 0x12340000
	 * 1: sha1check 0:off 1:on 
	 * 2: reset RC4 0:off 1:on 
	 * 3: FBCrypt 	0:off 1:HVM 2:PV
	 * 4: down		0:release 1:down
	 * 0000: 		encrypted data 
	 * --------------------------------*/

	/* Get flag */
	int	sha1chk  = (((p->data)>>28)&0x0f);
	int	resetKey = (((p->data)>>24)&0x0f);
	int	decFlag  = (((p->data)>>20)&0x0f);
	int	down	 = (((p->data)>>16)&0x0f);

	if ( KBD_LAYOUT_initFlag == 0 ) {
		init_vs2(&vs);
		KBD_LAYOUT_initFlag = 1;
		printk("init_vs OK\n");
	}

	/* RC4 state Reset */
	if ( resetKey ) { 
		RC4_reset(domid);
		/* 1. decrypt shared key*/
		/* 2. save shared key */
	}

	/* check Integrity */
	#if DEBUG_VKBD_INFO
	printk("sha1chk:%d\n",sha1chk);
	#endif
	if ( sha1chk ) { 
		if ( getSHA2((unsigned int)p->data,save_hash.seq,save_hash.hash) ) {
			p->data = 139; /* invalid input is "0" */
			return 1;
		}
	}
	
	if ( decFlag ) {
		/* decrypt key */
		decrypt = ps2_decrypt(domid,p->data,resetKey);

		/* convert to keycode */
		p->data = do_key_event(&vs,down,decrypt,1);

		#if DEBUG_VKBD_INFO
		printk("decrypt_sym: %c(%d)\n",(unsigned char)decrypt,decrypt);
		printk("keycode: %d\n",(unsigned int)p->data);
		#endif

	}
	return 0;
}

/* arch/x86/hvm/io.c hvm_io_assist */
void kbdMouse_info(ioreq_t *p)
{
	int domid = current_domain_id();
	static int memory = 0;
	#if TIME_HVM_KBD
    s_time_t start;
	s_time_t end;
	/* don't use double. VMM can't deal it */
	s_time_t overhead;
	static unsigned long logid = 0;
	#endif

	if ( p->addr == 100 ) {
		//printk("HVM DEBUG INFO Domain:%d ===================\n",domid);
		if ( p->data == 29 ) { 
		//	printk("[Keyboard] data:%ld(0x%08lx) addr:%ld +++++\n",p->data,p->data,p->addr);
			memory = p->data;
		}
		//if ( p->data == 61 ) { printk("[Mouse]:data:%ld(0x%08lx) addr:%ld    -----\n",p->data,p->data,p->addr); }
	} else if ( p->addr == 96 ) { 
		if ( memory == 29 ) {
			#if DEBUG_VKBD_INFO
			printk("\nHVM KBD INFO domain:%d ===================\n",domid);
			printk("value:%ld addr:%ld\n",p->data,p->addr); 
			#endif

			#if TIME_HVM_KBD
    		start = NOW();
			#endif

			hvm_decrypt(domid,p);
			memory = 0;

			#if TIME_HVM_KBD
    		end = NOW();
			overhead = end-start;
    		printk("ID:%ld HVM_INPUT_DECRYPT: %ld(ns)\n",logid++,overhead);
			#endif

		}
	}
}
