#include <asm/system.h>         /* hypercall */
#include <asm/current.h>        // current
#include <asm/page.h>           // virt_to_mfn
#include <xen/domain_page.h>    // map_domain_page
#include <xen/mm.h>             // alloc_domheap_pages
#include <xen/sched.h>          // struct domain
#include <xen/xmalloc.h>        // xmalloc_bytes
#include <public/io/fbif.h>     // struct xenfb_page
#include <xen/time.h>           // NOW()
#include "../cyassl/RC5/RC5_48.h"

#include <crypto/fbcrypt/pv/pv.h>

#define WIDTH_MAX 800
#define HEIGHT_MAX 600

struct dummy_reg
{   
    unsigned short domid;
    unsigned long pd_mfn;
    unsigned long *dummy_mfns;
    unsigned char *dummy_fb;

    rc5_ctx rc5[HEIGHT_MAX];
    int rounds;
    int keylen;
    unsigned char *key; 
    
    struct dummy_reg *next;
};

/* Dummy VFB for Domain0 */
struct dummy_reg *head;
//static int g_vfbFlag = 0;


/* shutdown時に呼ぶと落ちるけど新規ドメイン作成に問題はない */
/* 終了処理を完璧にするには考える必要があるけど問題ない */
void free_dummy_reg(struct domain *d)
{
    int i;
    struct dummy_reg *p1, *p2;
    p1 = p2 = head;

    for(i=0; p2 != NULL; i++)
    {
        if(p2->domid == d->domain_id)
        {
            printk("naonishi free0\n");
            if(i == 0){
                head = p2->next;
            }else{
                p1->next = p2->next;
            }

            free_domheap_pages(__virt_to_page(p2->dummy_mfns), 0);
            printk("naonishi free1\n");
            free_domheap_pages(__virt_to_page(p2->dummy_fb), 9);
            printk("naonishi free2\n");
            xfree((void *)p2);
            printk("naonishi free3\n");
            break;
        }
        p1 = p2;
        p2 = p2->next;
    }
}


/* create dummy VFB for Domain0 */
static void add_dummy_reg(struct domain *d, struct xenfb_page *page)
{
    int i;
    unsigned char *key_tmp;
    struct page_info *dummy_pg;
    struct dummy_reg *date;
    date = (struct dummy_reg *)xmalloc_bytes(sizeof(struct dummy_reg));

    date->domid = d->domain_id;
    date->pd_mfn = page->pd[0];
    
    dummy_pg = alloc_domheap_pages(d, 9, 1);
    date->dummy_fb = __page_to_virt(dummy_pg);
    memset(date->dummy_fb, 0, page->mem_length);

    dummy_pg = alloc_domheap_pages(d, 0, 1);
    date->dummy_mfns = __page_to_virt(dummy_pg);
    memset(date->dummy_mfns, 0, PAGE_SIZE);

    date->rounds = 16;
    date->key = (unsigned char *)"08237061"; /* notice */
    
    for(key_tmp=date->key, date->keylen=0; *key_tmp != '\0'; key_tmp++)
        date->keylen++;
    
	/* init RC5 */
    for(i=0; i<HEIGHT_MAX; i++){
        rc5_init(&(date->rc5[i]), date->rounds);
        rc5_key(&(date->rc5[i]), date->key, date->keylen, i);
    }

    date->next = head;
    head = date;
}


/* port は消していいかもしれん */
static inline int synchronize_vfb(struct xenfb_page *page, struct dummy_reg *p, int port)
{
    int x, y, offset;
    int i, j, k;
    int epd, nr_pages, mfns_num, mfns_end, page_per_pixels;
    unsigned long *mfns, *mfns_tmp;
    unsigned long *mfns_byte, *mfns_crypt;
    unsigned long crypt_words[2];
    union xenfb_out_event event;
	static unsigned int synchronize_flag = 0;
	#if TIME_PV_SCREEN
    s_time_t start;
	s_time_t end;
	/* don't use double. VMM can't deal it */
	s_time_t overhead;
	static unsigned long logid = 0;
	#endif

	
	/* Filtaling event */
	/* port==13 の時にIOリングが更新されているが
	 * port13の時のみでマスクすると暗号化がうまくいかないのはなぜ？ */
	if ( synchronize_flag == page->out_prod ) {
		//printk("synchronize_flag:%d\n",synchronize_flag);
		//printk("NOT synchronize\n");
		return 0;
	}

	synchronize_flag = page->out_prod;
	//printk("%ld:Synchronize-evtchn_port:%d\n",logid,port);
	//printk("in_cons:%d\n",page->in_cons);
	//printk("in_prod:%d\n",page->in_prod);
	//printk("out_prod:%d\n",page->out_prod);
	//printk("out_cons:%d\n",page->out_cons);
    
	/* IOリング参照 */
    event = XENFB_OUT_RING_REF(page, page->out_prod-1);

    
	/* IOリングから更新領域の座標取得 */
    if((event.type == XENFB_TYPE_UPDATE) &&
            ((event.update.x + event.update.width)<=WIDTH_MAX) &&
            ((event.update.y + event.update.height)<=HEIGHT_MAX)){

	#if TIME_PV_SCREEN
    start = NOW();
	#endif

    
    epd = PAGE_SIZE / sizeof(mfns[0]);
    nr_pages = (page->mem_length + PAGE_SIZE -1) >> PAGE_SHIFT;
    page_per_pixels = PAGE_SIZE/sizeof(unsigned int);
    
    mfns_byte = (unsigned long *)xmalloc_bytes(sizeof(unsigned long));
    memset(mfns_byte, 0, sizeof(unsigned long));
    mfns_crypt = (unsigned long *)xmalloc_bytes(PAGE_SIZE);
    memset(mfns_crypt, 0, PAGE_SIZE);
    
    
    //printk("%d\n",WIDTH_MAX*event.update.height);

    x = 0;
    y = event.update.y;
    mfns_num = (WIDTH_MAX*event.update.y)/page_per_pixels;
    offset = (WIDTH_MAX*event.update.y)%page_per_pixels;
    mfns_end = (WIDTH_MAX*(y+event.update.height))/page_per_pixels + 2;
    if(mfns_end >= nr_pages)
        mfns_end = nr_pages;

    for(i=0; i*epd < nr_pages; i++){
        mfns = map_domain_page(p->pd_mfn);
        for(j=mfns_num; j < mfns_end; j++){
            mfns_tmp = map_domain_page(mfns[j]);
            k = 0;
            if(j == mfns_num){
                k = offset/(sizeof(unsigned long)/sizeof(unsigned int));
                memcpy(mfns_crypt, p->dummy_fb + (i*epd+j)*PAGE_SIZE, k*sizeof(unsigned long));
            }
            for(k; k<(PAGE_SIZE/sizeof(unsigned long)); k++){
                
                memcpy(mfns_byte, mfns_tmp + k, sizeof(unsigned long));
                    
                crypt_words[0] = *mfns_byte & 0x0000000000ffffff;
                crypt_words[1] = *mfns_byte & 0x00ffffff00000000;
                crypt_words[1] >>= 32;
               
				/* 暗号化 */
                if(y < HEIGHT_MAX) {
                    rc5_encrypt(&(p->rc5[y]), crypt_words, 1, x);
				}
                    
                crypt_words[1] <<= 32;
                *mfns_byte = crypt_words[0] | crypt_words[1];
				//if (x==160) { printk("x:%d y:%d mfns_byte:%016lx\n",x,y,*mfns_byte);}
                
                memcpy(mfns_crypt + k, mfns_byte, sizeof(unsigned long));
            
                x += sizeof(unsigned long)/sizeof(unsigned int);
                if(x == WIDTH_MAX){
                    x = 0;
                    y++;
                }
            }
            memcpy(p->dummy_fb + (i*epd+j)*PAGE_SIZE, mfns_crypt, PAGE_SIZE);
            unmap_domain_page(mfns_tmp);
        }
        unmap_domain_page(mfns);
    }
    xfree((void *)mfns_byte);
    xfree((void *)mfns_crypt);
    
	#if TIME_PV_SCREEN
    end = NOW();
	overhead = (end-start)/1000; /* ns -> us */
	//if ( event.update.width == 800 && event.update.height == 600 ) {
    	printk("id:%ld: PV_VFB_ENCRYPT:%ld:(us) w:%d h:%d\n",
				logid++,overhead,event.update.width, event.update.height);
	//}
	#endif
    }
	return 0;
}


/* page構造体がDomain0に渡る */
static void init_fb(struct xenfb_page *page)
{
    int i;
    int epd, nr_pages;
    struct domain *d = current->domain;
   	
	MYLOG();
	printk("++++++++++++++++++++++++++++++++++\n");
	printk("domain_id:%d\n",d->domain_id);
	printk("++++++++++++++++++++++++++++++++++\n");
    add_dummy_reg(d, page);
 
    //synchronize_vfb(page, head);
    synchronize_vfb(page, head,0);

    epd = PAGE_SIZE / sizeof(head->dummy_mfns[0]);
    nr_pages = (page->mem_length + PAGE_SIZE -1) >> PAGE_SHIFT;

    for(i=0; i < nr_pages; i++)
        head->dummy_mfns[i] = __virt_to_mfn(head->dummy_fb + i*PAGE_SIZE);
    
	/* overwrite mfn to Domain0 */
	/* pd配列の中のmfnを書き換えている */
	/* 実際に使われているのはpd[0]のみ(800x600の場合のみ) */
    for(i=0; i*epd < nr_pages; i++)
        page->pd[i] = __virt_to_mfn(&(head->dummy_mfns[i*epd]));

    printk("naonishi pd[0]:%lu\n", page->pd[0]);
    printk("naonishi epd:%d nr_pages:%d\n", epd, nr_pages);
    printk("naonishi mem_length:%u\n", page->mem_length);
}

#if 0
/* init vfb */
int init_vfb_naonishi(int domid)
{
    struct xenfb_page *page = NULL;

    if ( g_vfbFlag == 0 ){
        head = NULL;
    	page = Domain_mfn2map_page(domid, VMM_VFB);
        init_fb(page);
        g_vfbFlag = 1;
        unmap_domain_page(page);
	}
	return 0;
}

/* synchronize_vfb */
int pub_synchronize_vfb(int domid)
{
    struct dummy_reg *p;
    struct xenfb_page *page = Domain_mfn2map_page(domid, VMM_VFB);

    if( g_vfbFlag == 1) {
        p = head;
        while ( p != NULL ) {
			/* DomainU のコンテキストなら VFB同期 */
            if(p->domid == domid) {
                synchronize_vfb(page, p);
                unmap_domain_page(page);
				return 0;
            }
            p = p->next;
        }
        printk("naonishi current_domid:%u\n",domid);
        printk("naonishi synchronize_vfb\n");

        init_fb(page); /* 2つめのDomainUのダミーVFB作成 */
    }

    unmap_domain_page(page);
	return 0;
}
#endif

//#if 0
/* evtchan に入れる */
//int send_vfb_mfn(int domid)
int send_vfb_mfn(int domid,int port)
{
    static int flag = 0;
    struct dummy_reg *p;
    struct xenfb_page *page = Domain_mfn2map_page(domid, VMM_VFB);

	/* notify Domain_mfn2map_pageはmfnが0でも何らかの値を返すのでNULLにならないことに注意 */
	if ( page == NULL ) return 0;

    if(flag == 0){ /* init */
        head = NULL;
        init_fb(page);
        flag = 1;
    }else if(flag == 1){
        p = head;
        
        while(p != NULL)
        {
			/* DomainU のコンテキストなら VFB同期 */
            if(p->domid == domid)
            {
                //synchronize_vfb(page, p);
                synchronize_vfb(page, p, port);
                unmap_domain_page(page);
				return 0;
            }
            p = p->next;
        }
        printk("naonishi current_domid:%u\n",domid);

        init_fb(page); /* 2つめのDomainUのダミーVFB作成 */
    }

    unmap_domain_page(page);
	return 0;
}
//#endif

//void send_vfb_mfn(unsigned long mfn)
//int send_vfb_mfn(void)
//{
//    static int flag = 0;
//    struct dummy_reg *p;
//    //struct xenfb_page *page = map_domain_page(mfn);
//    struct xenfb_page *page = Domain_mfn2map_page(domid, VMM_VFB);
//
//    if(flag == 0){ /* init */
//        head = NULL;
//        init_fb(page);
//        flag = 1;
//    }else if(flag == 1){
//        p = head;
//        
//        while(p != NULL)
//        {
//			/* DomainU のコンテキストなら VFB同期 */
//            if(p->domid == current->domain->domain_id)
//            {
//                synchronize_vfb(page, p);
//                unmap_domain_page(page);
//				return 0;
//            }
//            p = p->next;
//        }
//        printk("naonishi current_domid:%u\n", current->domain->domain_id);
//
//        init_fb(page); /* 2つめのDomainUのダミーVFB作成 */
//    }
//
//    unmap_domain_page(page);
//	return 0;
//}

