/*
 * common.c : FBCrypt sends input to VMM
 * 
 */

#include <time.h>
#include "../qemu-common.h"
#include "fbcrypt.h"

/* CASE:7 */
int keyEvent2VMM(int down, int seq, uint32_t sym)
{
	FBCRYPT_info fbc;

	V_TYPE mode  = ((sym>>20)&0x0f);
	int reset = ((sym>>24)&0x0f);
	int sha1Flag = ((sym>>28)&0x0f);

	/* fbc_info_table for PV tmp */
	fbc.down	= down;
	fbc.seq 	= seq;
	fbc.mode 	= ((sym>>20)&0x0f);
	fbc.reset 	= ((sym>>24)&0x0f);
	fbc.sha 	= ((sym>>28)&0x0f);

	if ( mode == HVM ) {
		#if COMMON_DEBUG_KBD
		printf("\nFBCRYPT_HVM ------------------\n");
		printf("sym:%d(0x%08x)\n",sym,sym);
		printf("down:%d seq:%d reset:%d sha1:%d\n",down,seq,reset,sha1Flag);
		#endif
		HVM_keyEvent2VMM(down,sym);
		
	} else if ( mode == PV ) {
		#if COMMON_DEBUG_KBD
		printf("\nFBCRYPT_PV ------------------\n");
		printf("sym:%d(0x%08x)\n",sym,sym);
		printf("down:%d seq:%d reset:%d sha1:%d\n",down,seq,reset,sha1Flag);
		#endif
		PV_keyEvent2VMM(sym,&fbc);
	}
	return 0;	
}

/* CASE:8 check SHA1 mode */
int SHA_keyEvent2VMM(int down, int seq, uint32_t sym, 
		uint32_t hash0, uint32_t hash1, uint32_t hash2, uint32_t hash3, uint32_t hash4)
{
	FBCRYPT_info fbc;

	V_TYPE mode  = ((sym>>20)&0x0f);
	int reset = ((sym>>24)&0x0f);
	int sha1Flag = ((sym>>28)&0x0f);

	/* fbc_info_table for PV tmp */
	fbc.down	= down;
	fbc.seq 	= seq;
	fbc.mode 	= ((sym>>20)&0x0f);
	fbc.reset 	= ((sym>>24)&0x0f);
	fbc.sha 	= ((sym>>28)&0x0f);
	fbc.hash[0] = hash0; fbc.hash[1] = hash1;
	fbc.hash[2] = hash2; fbc.hash[3] = hash3;
	fbc.hash[4] = hash4;

	if ( mode == HVM ) {
		#if COMMON_DEBUG_KBD
		printf("\nFBCRYPT_HVM ------------------\n");
		printf("sym:%d(0x%08x)\n",sym,sym);
		printf("down:%d seq:%d reset:%d sha1:%d\n",down,seq,reset,sha1Flag);
		if ( sha1Flag ) {
			printf("hash0:%d(0x%08x)\n",hash0,hash0);
			printf("hash1:%d(0x%08x)\n",hash1,hash1);
			printf("hash2:%d(0x%08x)\n",hash2,hash2);
			printf("hash3:%d(0x%08x)\n",hash3,hash3);
			printf("hash4:%d(0x%08x)\n",hash4,hash4);
		}
		#endif
		SHA_HVM_keyEvent2VMM(down,sym,&fbc);
		
	} else if ( mode == PV ) {
		#if COMMON_DEBUG_KBD
		printf("\nFBCRYPT_PV ------------------\n");
		printf("sym:%d(0x%08x)\n",sym,sym);
		printf("down:%d seq:%d reset:%d sha1:%d\n",down,seq,reset,sha1Flag);
		if ( sha1Flag ) {
			printf("hash0:%d(0x%08x)\n",hash0,hash0);
			printf("hash1:%d(0x%08x)\n",hash1,hash1);
			printf("hash2:%d(0x%08x)\n",hash2,hash2);
			printf("hash3:%d(0x%08x)\n",hash3,hash3);
			printf("hash4:%d(0x%08x)\n",hash4,hash4);
		}
		#endif
		PV_keyEvent2VMM(sym,&fbc);
	}
	return 0;	
}

/* HYPER_CALL No.43 */
void HC_CHECK_PRINT(unsigned char *s)
{
	//my_hypercall_v1(__HYPERVISOR_check_print,(void *)s);
	hypercall_v2(__HYPERVISOR_check_print,1,(void *)s);
}

/* HYPER_CALL No.42 */
int HC_get_time_now(int stop)
{
	/* flag: 0 start 
	 * flag: 1 stop
	 */
	//int ret = my_hypercall_v1(__HYPERVISOR_get_time_now,(void *)&stop);
	int ret = hypercall_v2(__HYPERVISOR_get_time_now,1,(void *)&stop);
	if (ret) printf("Miss do_get_time_now \n");
	return 0;
}


/* ------------------------------------------- My KEY LOGGER */
int vnc_keylogger2(unsigned int buf)
{
	FILE *fp = fopen("/home/egawa/keylog.txt","a");
	fprintf(fp,"%c ",(unsigned char)buf);
	fclose(fp);
	return 0;
}


/* ------------------------------------------- TIME-LOG */
int my_timelog(double time)
{
	FILE *fp = fopen("/home/egawa/LOG/QEMU_HC_DECRYPT.txt","a");
	double overhead = time*1000;
	fprintf(fp,"%.4f:(msec)\n",overhead);
	fclose(fp);
	return 0;
}


int my_timelog2(double time)
{
	FILE *fp = fopen("/home/egawa/LOG/QEMU_SERVER_RES.txt","a");
	double overhead = time*1000;
	fprintf(fp,"%.4f:(msec)\n",overhead);
	fclose(fp);
	return 0;
}


double my_gettimeofday_sec(void)
{
	struct timeval tv;
	gettimeofday(&tv,NULL);
	return tv.tv_sec + (double)tv.tv_usec*1e-6;
}
/* END ------------------------------------------- TIME-LOG */

